'''
Created on 09-06-2012

@author: firemark

Formatting IRC messages.
Colors, bolding...
'''


def bold(msg):
    return "\x02" + msg + "\x02"


def underline(msg):
    return "\x1F" + msg + "\x1F"


def italic(msg):
    return "\x16" + msg + "\x16"


def color(msg, front, back=None):
    if back:
        front = str(front) + ',' + str(back)
    else:
        front = str(front)
    return "\x03" + front + msg + "\x03"

#colors
color.white = 0
color.black = 1
color.dark_blue = 2
color.green = 3
color.red = 4
color.brown = 5
color.purple = 6
color.olive = 7
color.yellow = 8
color.light_green = 9
color.teal = 10
color.cyan = 11
color.light_blue = 12
color.pink = 13
color.dark_gray = 14
color.light_gray = 15



from .models import WhoisFuture

import asyncio


class IrcSender(object):

    transport = None
    whois_heap = None

    def __init__(self):
        self.whois_heap = {}

    def send(self, *args):
        string = ' '.join(str(arg) for arg in args)[:510] + '\r\n'
        self.transport.write(string.encode())

    def send_msg(self, *args):
        """add ':' to last argument"""
        new_args = list(args)
        new_args[-1] = ':' + str(new_args[-1])
        self.send(*new_args)

    def say(self, nick_chan, msg):
        self.send_msg('PRIVMSG', nick_chan, msg)

    def notice(self, nick, msg):
        self.send_msg('NOTICE', nick, msg)

    def reply(self, nick, msg, chan=None):
        msg = msg if not chan else '%s: %s' % (nick, msg)
        self.say(chan or nick, msg)

    def kick(self, nick, chan, why=''):
        self.send_msg('KICK', chan, nick, why or nick)

    def mode(self, chan, flag, arg):
        self.send('MODE', chan, flag, arg)

    @asyncio.coroutine
    def whois(self, nick):
        """

        :type nick: str
        :return: WhoisData
        """
        self.whois_heap[nick] = future = WhoisFuture()
        self.send('WHOIS', nick)
        result = yield from future
        return result

    def time_ban(self, time, nick, chan, why='', prefix=None):
        """

        :type time: int
        :type nick: str
        :type chan: str
        :type why: str
        :type prefix: str or NoneType
        """

        if time <= 0:
            return

        prefix = prefix or nick + '!*@*'
        self.mode(chan, '+b', prefix)
        self.kick(nick, chan, why)

        @asyncio.coroutine
        def timer():
            yield from asyncio.sleep(time)
            self.mode(chan, '-b', prefix)

        asyncio.async(timer())



#!/usr/bin/python3

from ..utils import create_help, register
from math import isnan
from random import random
from ..utils.types import range_int
from .. import format
import operator
import math

special_vars = {
    'pi': math.pi,
    'e': math.e
}

operator_funcs = {
    '+': operator.add,
    '-': operator.sub,
    '/': operator.truediv,
    '*': operator.mul,
    '%': operator.mod,
    '^': operator.pow,
    '=': operator.eq,
    '>': operator.gt,
    '<': operator.lt,
    '&': lambda a, b: bool(a) and bool(b),
    '|': lambda a, b: bool(a) or bool(b),
}

one_var_operator_funcs = {
    '~': lambda a: not bool(a),
    'sin': math.sin,
    'cos': math.cos,
    'log': math.log,
    'log10': math.log10,
    'log2': math.log2,
}


@register(cmd="onp")
def onp(bot, tasks, round:range_int(0)=3):
    """
        example: .onp 2 2 2 + * log2 round=5
        operators https://github.com/firemark/grazyna/blob/master/grazyna/plugins/onp.py
    """
    score = calc(tasks.split(), round)
    bot.say("Result: %s" % score)


def calc(tasks, round_num=3):
    buffer = []
    for task in tasks:
        num = get_number(task)
        if num is None:
            result = execute(task, buffer)
            if result is not None:
                return result
        else:
            buffer.append(num)

    if len(buffer) > 0:
        value = buffer.pop()
        if type(value) == bool:
            return value.__repr__()
        else:
            if isnan(value):
                if random() > 0.87:
                    return "NaNNaNNaNNaNNaN Batman!"
            return round(value, round_num)
    else:
        return "WTF"


def get_number(task):
    try:
        return float(task)
    except ValueError:
        try:
            return special_vars[task]
        except KeyError:
            return None


def execute(op, buffer):
    func = operator_funcs.get(op)
    if func is None:
        func = one_var_operator_funcs.get(op)
        try:
            args = [buffer.pop()]
        except IndexError:
            return "WTF"
    else:
        try:
            args = [buffer.pop(), buffer.pop()]
        except IndexError:
            return "WTF"
    try:
        value = func(*args)
    except OverflowError:
        return format.bold("Over 9000!!")
    else:
        buffer.append(value)



from grazyna.utils import register
from grazyna import format

from aiohttp import request

GOOGLE_URL = 'https://ajax.googleapis.com/ajax/services/search/web'


@register(cmd='g')
def google(bot, query):
    """get first result of google query"""

    response = yield from request('GET', GOOGLE_URL, params={
        'q': query,
        'v': '1.0'
    })
    json_data = yield from response.json()
    data = json_data['responseData']
    if data is None:
        bot.reply(response['responseDetails'])
        return
    try:
        result = data['results'][0]
    except IndexError:
        bot.reply('not found')
        return

    title = result['titleNoFormatting']
    url = result['url']
    bot.say('{}: {}'.format(format.bold(title), url))



#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""
ruletka ^_^
"""
from ..utils import register, create_help, init_plugin
from random import randint
from .. import format
from datetime import datetime, timedelta


ban_times = (5, 10, 13, 14, 15, 15)

create_help('ruletka', '.krec || .strzal')


@init_plugin
def init(plugin, protocol):
    plugin.temp.update(
        on_play=False,
        last_nick=None,
        last_warn=datetime.now()
    )


@register(cmd='krec', on_private=False)
def play(bot):
    if not bot.temp['on_play']:
        bot.temp.update(
            cell=0,
            cell_with_bullet=randint(0, 5),
            last_warn=datetime.now(),
            on_play=True,
        )
        bot.say(format.bold('*trrrr*'))
    else:
        bot.say('zakręcone!')


@register(cmd='strzal', on_private=False)
def shot(bot):
    if not bot.temp.get('on_play', False):
        return

    old_last_warn = bot.temp['last_warn']
    bot.temp['last_warn'] = datetime.now()

    if bot.temp['last_nick'] == bot.user.prefix:
        if datetime.now() - old_last_warn < timedelta(minutes=1):
            bot.reply('Spokoj!')
        return

    cell = bot.temp['cell']
    if bot.temp['cell_with_bullet'] == cell:
        bot.time_ban(ban_times[cell], why='Bach!')
        bot.temp.update(
            last_nick=None,
            on_play=False
        )
    else:
        bot.temp['cell'] += 1
        bot.temp['last_nick'] = bot.user.prefix
        bot.say(format.bold('*klik*'))



#!/usr/bin/python3
from ..utils import register
from .. import format

from inspect import getfullargspec
import re


@register(cmd='help')
def cmd_help(bot, name=None):
    """show help"""
    importer = bot.protocol.importer
    if name is None:
        show_commands(bot, importer)
    else:
        show_command_help(bot, importer, name)


@register(cmd='args')
def cmd_args(bot, name):
    """show args"""
    importer = bot.protocol.importer
    plugin, func = importer.find_command(name)
    if func is None:
        return bot.reply('cmd not found')
    cmd = func.cmd.format(**importer.get_plugin_cfg(plugin.name))
    argspec = getfullargspec(func)
    args = argspec.args[1:]
    defaults = argspec.defaults
    kwonlyargs = argspec.kwonlyargs
    bot.say(
        '{cmd}: {args} {optional} {very_optional}'.format(
            cmd=format.bold(cmd),
            args=' '.join(args[:-len(defaults)]),
            optional=' '.join('[%s]' % a for a in args[-len(defaults):]),
            very_optional=' '.join('[%s]' % a for a in kwonlyargs),
        )
    )

@register(cmd='source')
def source(bot):
    bot.say('https://github.com/firemark/grazyna')


def show_commands(bot, importer):
    bot.say(
        ', '.join(
            func.cmd.format(**importer.get_plugin_cfg(plugin.name))
            for plugin, func in importer.get_commands()
            if not func.admin_required
        )
    )


def show_command_help(bot, importer, name):
    plugin, func = importer.find_command(name)
    if func is None:
        return bot.reply('cmd not found')

    cmd = func.cmd.format(**importer.get_plugin_cfg(plugin.name))
    doc = re.sub('\s+', ' ', func.__doc__) if func.__doc__ else 'help not found'
    bot.say('{}: {}'.format(format.bold(cmd), doc))



#class commands(Plugin):
#    reg = r'^\.commands'
#    help = '.commands'
#    name = 'commands'
#
#    def use(self, input):
#        cmds = []
#        for module in modules.values():
#            cmds += [
#                      plugin.name
#                      for _, plugin in module
#                      if plugin.help
#                      ]
#            cmds.sort()
#        self.say(format.bold('commands: ') + ', '.join(cmds) )


from grazyna.utils import register
from grazyna import format
from grazyna.format import color
from datetime import timedelta, datetime, timezone

from aiohttp import request
import re

URL_API = 'http://api.openweathermap.org/data/2.5/forecast'
URL_API_LONG = URL_API + '/daily'

re_date = re.compile(r'([+-]?\d+)([dh])')

ICON_TO_UTF = {
    '01d': color('☀', color.yellow),
    '02d': color('☀', color.yellow),
    '03d': color('☁', color.white),
    '04d': color('☁', color.white),
    '09d': color('☔', color.light_blue),
    '10d': color('☔', color.light_blue),
    '11d': color('⚡', color.yellow),
    '13d': color('❄', color.white),
    '01n': color('☾', color.white),
    '02n': color('☾', color.white),
    '03n': color('☁', color.white),
    '04n': color('☁', color.white),
    '09n': color('☔', color.light_blue),
    '10n': color('☔', color.light_blue),
    '11n': color('⚡', color.yellow),
    '13n': color('❄', color.white),
}

@register(cmd='weather')
def weather(bot, city, day=None):
    dt = check_and_return_datetime(day)
    timestamp = dt.replace(tzinfo=timezone.utc).timestamp()

    delta_days = (dt - datetime.now()).days
    is_long_forecast = (delta_days > 5)
    if not is_long_forecast:
        response = yield from request('GET', URL_API, params={
            'q': city,
            'units': 'metric',
            'lang': 'pl',
            'APPID': bot.config.get('app_id'),
        })
    else:
        response = yield from request('GET', URL_API_LONG, params={
            'q': city,
            'units': 'metric',
            'lang': 'pl',
            'cnt': min(delta_days, 16),
            'APPID': bot.config.get('app_id'),
        })
    try:
        json_data = yield from response.json()
    finally:
        response.close()
    if json_data.get('cod') == 401:
        return
    data_list = json_data["list"]
    data = sorted(data_list, key=lambda x: abs(x['dt'] - timestamp))[0]
    try:
        weather = data['weather'][0]
    except IndexError:
        weather = {'description': '', 'icon': ''}

    if not is_long_forecast:
        temperature = data['main']['temp']
    else:
        if dt.hour < 6:
            period = 'night'
        elif dt.hour < 10:
            period = 'morn'
        elif dt.hour < 18:
            period = 'day'
        elif dt.hour < 22:
            period = 'eve'
        else:
            period = 'night'
        temperature = data['temp'][period]
    bot.reply('{city}: {temp} °C {icon} {desc}'.format(
        city=format.bold(city),
        temp=temperature,
        desc=weather['description'],
        icon=ICON_TO_UTF.get(weather['icon'], '')
    ))


def check_and_return_datetime(day):
    dt = datetime.now()
    if day is None:
        return dt

    dt_hour = datetime.now().replace(hour=12, minute=0, second=0)
    if day == "yesterday":
        return dt_hour - timedelta(days=1)
    if day == "tomorrow":
        return dt_hour + timedelta(days=1)

    match = re_date.match(day)
    if match is None:
        return dt
    number = int(match.group(1))
    if match.group(2) == 'd':
        return dt_hour + timedelta(days=number)
    else:
        return dt + timedelta(hours=number)



from datetime import datetime
from aiohttp import ClientSession
from grazyna.utils import register
from grazyna.utils.types import range_int
from lxml.html import fromstring as fromstring_to_html

import asyncio
import dateparser
import re

URL = 'https://wiki.hs-silesia.pl/wiki'
URL_MEETS = '%s/Planowane_spotkania' % URL
URL_CYCLIC = '%s/Cykliczne_obowiązki' % URL
TRASH_ID = "Wyw.C3.B3z_.C5.9Bmieci"
MEETS_ID = "Najbli.C5.BCsze_spotkania_tematyczne"
PREV_MEETS_ID = "Odbyte_spotkania"
LIST_XPATH = '//span[starts-with(@id,$id)]/../following-sibling::ul[1]/li'

re_date_range = re.compile(r"(\d\d+?)\s*-\s*(\d\d?)")
re_every = re.compile(r"^[Kk]a[żz]d[yaą]")

@asyncio.coroutine
def get_html(url):
    session = ClientSession()
    try:
        resp = yield from session.get(url)
        try:
            raw_data = yield from resp.read()
            return fromstring_to_html(
                raw_data.decode('utf-8', errors='xmlcharrefreplace')
            )
        finally:
            resp.close()
    finally:
        session.close()


def get_list_of_text_sorted_by_time(nodes, parse_date=None, old_dates=False):
    parse_date = parse_date or (lambda text: dateparser.parse(text))
    now = datetime.now()
    list_data = sorted(
        (
            (text, date) for text, date in (
                (text.strip(), parse_date(text))
                for text in (node.text for node in nodes) if text
            )
            if date is not None and (
                (not old_dates and date > now) or
                (old_dates and date < now)
            )
        ),
        key=lambda obj: (obj[1] - now) * (not old_dates * 2 - 1)  # 0,1 -> 1,-1
    )
    return [text for text, date in list_data]


def meet_parse_date(text):
    text = re_date_range.sub(r'\2', text)
    raw_date, _, _ = text.partition('-')
    raw_date = raw_date.strip()
    raw_date = re_every.sub('', raw_date)
    return dateparser.parse(raw_date)


@asyncio.coroutine
def show_meets(bot, position, label, old_dates):
    html = yield from get_html(URL_MEETS)
    nodes = html.xpath(LIST_XPATH, id=label)
    list_data = get_list_of_text_sorted_by_time(
        nodes, meet_parse_date, old_dates)
    if not list_data:
        return bot.reply('¬_¬ no meetings')
    len_list = len(list_data)
    try:
        text = list_data[position - 1]
        text += " [%d meets]" % len_list
    except IndexError:
        text = 'only %d meets in calendar!' % len_list
    bot.reply(text)


@register(cmd='next-meet')
def next_meet(bot, position: range_int(1)=1):
    yield from show_meets(bot, position, MEETS_ID, old_dates=False)


@register(cmd='prev-meet')
def prev_meet(bot, position: range_int(1)=1):
    yield from show_meets(bot, position, PREV_MEETS_ID, old_dates=True)


@register(cmd='trash')
def trash(bot):
    html = yield from get_html(URL_CYCLIC)
    nodes = html.xpath(LIST_XPATH, id=TRASH_ID)
    list_data = get_list_of_text_sorted_by_time(nodes)
    if not list_data:
        return bot.reply('czysto i pięknie, uzupełnij wiki.')
    bot.reply(list_data[0])



from sys import argv
from grazyna.config import create_config
from grazyna.irc import client


def run():
    with open(argv[1]) as f:
        cfg = create_config(f)
    client.connect(cfg)

if __name__ == "__main__":
    run()
